/* Returns Size of Directories */

	/* Standard 'C' Includes */
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <dir.h>
#include <dos.h>
#include <string.h>

	/* User Defined Includes */
#include "dirsize.h"

typedef enum { FALSE = 0, TRUE = 1 } boolean;

	/* What 'level' in the dir structure are we in? */
int level = 0;

	/* Number of Lines Outputed */
int num_lines = 0;

	/* Total Number of Files and Directories */
int num_files = 0;
int num_dirs = 0;

	/* Various Options */
int num_levels = 1;
boolean pause = FALSE;

	/* Temp variables */
long temp_size = 0;
char buffer[20];


/*========== Pauses the Output ============================================*/
void pause_output (void) {
  num_lines = 0;
  printf ("Press any Key to Continue...");
  getch();
  delline();
  gotoxy (1, wherey());
 }
/*========== End of Procedure pause_output() ==============================*/

/*========== Displays a Basic Help Screen =================================*/
void display_help (void) {
  printf ("\n");
  printf ("DIRSIZE - v0.10 - Dave Humphrey 1998\n");
  printf ("   Displays size of directory tree. Useful in determining what\n");
  printf ("   directories are taking up the most space.\n\n");

  printf ("   Options:\n");
  printf ("      /P ...... Pauses display output every %d lines.\n", MAX_LINES);
  printf ("      /L# ..... Only outputs # levels of the directory tree.\n");
  printf ("                Default is only one level.\n\n");
  printf ("Any bugs/comments etc... can be sent to aj589@freenet.carleton.ca\n");
 }
/*========== End of Procedure display_help() ==============================*/

/*========== Main Sizing Function =========================================*/
long find_size (void) {
  struct ffblk file_block;
  long size = 0;
  int done = FALSE;
  int temp_files;

  done = findfirst ("*.*", &file_block, FA_DIREC);

	/* Search Through all Files... */
  while (!done) {

			/* Is the file a directory */
	if (file_block.ff_attrib == FA_DIREC) {

			/* The 'special' directories don't count... */
	  if (file_block.ff_name[0] == '.' && (!file_block.ff_name[1] || file_block.ff_name[1] == '.')) {
			/* Do Nothing */
	   }
	  else {
			/* Attempt to change to this dir */
		if (!chdir(file_block.ff_name)) {
		  num_dirs++;
		  temp_files = num_files;
		  level++;

		  if (level < num_levels) {
			printf ("%*s \\%s\n", level<<2, " ", file_block.ff_name);
			num_lines++;
			if (pause && num_lines >= MAX_LINES) pause_output();
		   }
		  else if (level == num_levels)
			printf ("%*s \\%s", level<<2, " ", file_block.ff_name);

		  temp_size = find_size ();
		  size += temp_size;

		  if (level < num_levels) {
			printf ("%*s  Total = %s bytes, %d files\n", level<<2, " ", print_number(temp_size), num_files - temp_files);
			num_lines++;
			if (pause && num_lines >= MAX_LINES) pause_output();
		   }
		  else if (level == num_levels) {
			printf (" (%s bytes, %d files)\n", print_number(temp_size), num_files - temp_files);
			num_lines++;
			if (pause && num_lines >= MAX_LINES) pause_output();
		   }

		  level--;
		  chdir ("..");
		 }
	   }
	 }
	else {	/* Must be a file, add to size */
	  size += file_block.ff_fsize;
	  num_files++;
	 }

	done = findnext (&file_block);
   }

  return (size);
 }
/*========== End of Function find_size() ==================================*/


/*========== Prints a Comma Demlimited Number =============================*/
char *print_number (const long l) {
  char buffer1[20];
  register int i, j, k;

  sprintf (buffer1, "%ld", l);
  j = strlen(buffer1) - 1;

  for (i = j, k = 0; i >= 0; i--) {
	buffer[k] = buffer1[i];
	k++;

	if ((k + 1) % 4 == 0 && i) {
	  buffer[k] = ',';
	  k++;
	 }
   }

  buffer[k] = 0;
  return (strrev(buffer));
 }
/*========== End of Procedure print_number() ==============================*/

/*========== Begin Main Program ===========================================*/
void main (int argv, char *argc[]) {
  char cur_dir[101];	/* Save the Current Directory */
  int i;		/* Loop counter */
  long s;		/* Temp Variable */

		/* Save the Current Dir */
  getcwd (cur_dir, 100);

		/* Parse the Command Line Parameters */
  for (i = 1; i < argv; i++) {
	strlwr(argc[i]);

	if (!stricmp(argc[i], "/p"))
	  pause = TRUE;
	else if (!strnicmp(argc[i], "/l", 2))
	  num_levels = atoi(argc[i] + 2);
	else {
	  display_help();
	  exit (FALSE);
	 }
   }

  printf ("\nStarting From Directory %s\n", cur_dir);
  s = find_size();
  printf ("Totals: %s Bytes,  %d Files,  %d Directories\n", print_number(s), num_files, num_dirs);

		/* Return to Original Dir */
  chdir (cur_dir);
 }
/*========== End of Main Program ==========================================*/